@@ -172,7 +172,19 @@ def load_configuration():
     CONFIG.set('management','ss_key'     , getenv('QGSRV_MANAGEMENT_SSL_KEY' ,''))
     CONFIG.set('management','ssl_cert'   , getenv('QGSRV_MANAGEMENT_SSL_CERT',''))
     CONFIG.set('management','port'       , getenv('QGSRV_MANAGEMENT_PORT','19876'))
-  
+ 
+    #
+    # Metadata
+    #
+    CONFIG.add_section('metadata')
+    CONFIG.set('metadata', 'contact_name', '3liz')
+    CONFIG.set('metadata', 'contact_address', '')
+    CONFIG.set('metadata', 'contact_email', 'info@3liz.com')
+    CONFIG.set('metadata', 'contact_url', 'https://www.3liz.com/')
+    CONFIG.set('metadata', 'licence_url', 'http://mozilla.org/MPL/2.0/')
+    CONFIG.set('metadata', 'licence_name','Mozilla Public Licences, v2.0')
+    CONFIG.set('metadata', 'external_doc_description','Py-qgis-wps server documentation')
+    CONFIG.set('metadata', 'external_doc_url','https://docs.3liz.org/py-qgis-wps/')
 
 def read_config_file( cfgfile ):
     """ Read configuration from file
