@@ -24,33 +24,27 @@ LOGGER = logging.getLogger('SRVLOG')
 class CacheCollection(RequestHandler):
 
     def get(self, key: str=None) -> None: 
-        """ Return plugin info
+        """ Return project cache info
         """
         if not key:
-            raise HTTPError(403)
+            # Try to get key from param
+            key = self.request.parameter('MAP')
+
+        if not key:
+            raise HTTPError(400,reason="Missing project specification")
 
-        # We don't want to update the cache
-        # Unless the project is not loaded
         cache = get_cacheservice()
-        try:
-            project, _, _ = cache.get_project(key, strict=False)
-        except cache.UnreadableResourceError:
-            raise HTTPError(422,reason=f"Cannot read project resource '{key}'") from None
-        except cache.PathNotAllowedError:
-            raise HTTPError(403,reason="Project path not allowed") from None
-        except FileNotFoundError:
-            raise HTTPError(404,reason=f"Project '{key}' not found") from None
-        
+        project, _ = cache.lookup(key, refresh=False)
+ 
         self.write(get_project_summary(key, project))  
-                
-        
+    
+ 
 def register( serverIface ):
     """ Register plugins api handlers
     """
     register_handlers(serverIface, "/cache","CacheManagment",
                       [
-                          (r'/(?P<key>.+)$', CacheCollection),
+                          (r'/content/(?P<key>.+)$', CacheCollection),
                           (r'/', CacheCollection),
                       ])
-                      
 
