@@ -18,8 +18,6 @@ from typing import Optional
 from pyqgisserver.plugins import plugin_list, plugin_metadata, failed_plugins
 from .handler import RequestHandler, register_handlers
 
-from qgis.core import Qgis
-
 LOGGER = logging.getLogger('SRVLOG')
 
 
@@ -38,22 +36,26 @@ class PluginCollection(RequestHandler):
                 raise HTTPError(404)
             self.write({'name': name, 'status': 'loaded', 'metadata': metadata })
         else:
-            def link(name):
-                return self.public_url(f"/{name}")
+            def _link(name, status):
+                return { 
+                    'href': self.public_url(f"/{name}"),
+                    'status': status,
+                    'name': name,
+                    'type': 'application/json',
+                    'title': f'Details for plugin {name}',
+                }
             # List all loaded plugins
-            plugins = [{ 'name': n, 'status': 'loaded', 'link': link(n) } for n in plugin_list()]
-            plugins.extend([{ 'name' : n, 'status': 'failed', 'link': link(n) } for n in failed_plugins])
-            self.write({'plugins': plugins })
+            plugins = [_link(n,'loaded') for n in plugin_list()]
+            plugins.extend([_link(n,'failed') for n in failed_plugins])
+            self.write({'links': plugins })
 
         
 def register( serverIface ):
     """ Register plugins api handlers
     """
-    # XXX See https://github.com/qgis/QGIS/issues/45439
-    prefix = '/plugins' if Qgis.QGIS_VERSION_INT < 32200 else ''
     register_handlers(serverIface, "/plugins","PluginsManagment",
                       [
-                          (rf'{prefix}/(?P<name>[^\/]+)/?$', PluginCollection),
+                          (r'/(?P<name>[^\/]+)/?$', PluginCollection),
                           (r'/', PluginCollection),
                       ])
 
