@@ -15,6 +15,8 @@ import signal
 import tornado.web
 import tornado.platform.asyncio
 
+from tornado.web import RedirectHandler
+
 from multiprocessing import Process
 
 from .logger import log_request
@@ -22,10 +24,10 @@ from .config import confservice, qgis_api_endpoints
 
 from .handlers import (StatusHandler, 
                        OwsHandler, 
-                       OwsApiHandler,
+                       OAPIHandler,
                        PingHandler, 
-                       NotFoundHandler,
-                       ErrorHandler)
+                       LandingPage,
+                       NotFoundHandler)
 
 from .zeromq import client, broker
 
@@ -50,14 +52,13 @@ def configure_handlers( client: client.AsyncClient ) -> [tornado.web.RequestHand
         client       = client,
         monitor      = monitor,
         timeout      = cfg.getint('timeout'),
-        http_proxy   = cfg.getboolean('http_proxy'),
         allowed_hdrs = tuple(k.upper() for k in cfg.get('allow_headers').split(','))
     )
 
     end = r"(?:\.html|\.json|/?)"
 
     handlers = [
-        (r"/", ErrorHandler, dict(status_code=403)),
+        (r"/", LandingPage),
         (r"/ping", PingHandler),
     ]
 
@@ -66,14 +67,14 @@ def configure_handlers( client: client.AsyncClient ) -> [tornado.web.RequestHand
 
     # Server status page
     if cfg.getboolean('status_page'):
-        handlers.append( ("/status/?", StatusHandler) )
+        handlers.append( ("/status/?.*", StatusHandler) )
 
     def _ows_args( *args, **kwargs ):
         rv = ows_kwargs.copy()
         rv.update( *args, **kwargs )
         return rv
 
-    add_handler( r"/ows/?", OwsHandler, _ows_args(getfeaturelimit=cfg.getint('getfeaturelimit')))
+    add_handler(r"/ows/?", OwsHandler, _ows_args(getfeaturelimit=cfg.getint('getfeaturelimit')))
 
     wfs3_api_endpoints = [
         rf"{end}",
@@ -84,14 +85,12 @@ def configure_handlers( client: client.AsyncClient ) -> [tornado.web.RequestHand
     ] 
 
     # XXX DEPRECATED (to be removed in 1.9)
-    kw = _ows_args(service='WFS3')
-    for endpoint in wfs3_api_endpoints:
-        handlers.append( (rf"/ows/wfs3{endpoint}", OwsApiHandler, kw) )
+    handlers.append((rf"/ows/wfs3({end}.*)", RedirectHandler, {'url': "/wfs3{0}" }))
 
     # New scheme
     kw = _ows_args(service='WFS3')
     for endpoint in wfs3_api_endpoints:
-        handlers.append( (rf"/wfs3{endpoint}", OwsApiHandler, kw) )
+        handlers.append((rf"/wfs3{endpoint}", OAPIHandler, kw) )
 
     #
     # Add qgis api endpoints
@@ -99,7 +98,7 @@ def configure_handlers( client: client.AsyncClient ) -> [tornado.web.RequestHand
     for name, endpoint in qgis_api_endpoints():
         kw = _ows_args(service=name)
         LOGGER.debug("*** Adding API handler for: %s: %s", name, endpoint)        
-        handlers.append( (rf"/{endpoint.strip('/')}/.*", OwsApiHandler, kw) )
+        handlers.append( (rf"/{endpoint.strip('/')}/.*", OAPIHandler, kw) )
 
     return handlers
 
@@ -115,6 +114,8 @@ class Application(tornado.web.Application):
         self._broker_client = client.AsyncClient(router, bytes(identity.encode('ascii')))
         self.stats = Stats()
 
+        self.http_proxy = confservice.getboolean('server','http_proxy')
+
         super().__init__(configure_handlers(self._broker_client),
                          default_handler_class=NotFoundHandler)
 
